@@ -79,6 +79,8 @@ static o2fsck_state _ost;
 static int cluster_locked = 0;
 
 static void mark_magical_clusters(o2fsck_state *ost);
+static errcode_t fsck_lock_fs(o2fsck_state *ost);
+static void fsck_unlock_fs(o2fsck_state *ost);
 
 static void handle_signal(int sig)
 {
@@ -202,17 +204,17 @@ errcode_t o2fsck_state_reinit(ocfs2_filesys *fs, o2fsck_state *ost)
 {
 	errcode_t ret;
 
-	ocfs2_bitmap_free(ost->ost_dir_inodes);
+	ocfs2_bitmap_free(&ost->ost_dir_inodes);
 	ost->ost_dir_inodes = NULL;
 
-	ocfs2_bitmap_free(ost->ost_reg_inodes);
+	ocfs2_bitmap_free(&ost->ost_reg_inodes);
 	ost->ost_reg_inodes = NULL;
 
-	ocfs2_bitmap_free(ost->ost_allocated_clusters);
+	ocfs2_bitmap_free(&ost->ost_allocated_clusters);
 	ost->ost_allocated_clusters = NULL;
 
 	if (ost->ost_duplicate_clusters) {
-		ocfs2_bitmap_free(ost->ost_duplicate_clusters);
+		ocfs2_bitmap_free(&ost->ost_duplicate_clusters);
 		ost->ost_duplicate_clusters = NULL;
 	}
 
@@ -232,6 +234,22 @@ errcode_t o2fsck_state_reinit(ocfs2_filesys *fs, o2fsck_state *ost)
 	return 0;
 }
 
+static void o2fsck_state_release(o2fsck_state *ost)
+{
+	if (ost->ost_dir_inodes)
+		ocfs2_bitmap_free(&ost->ost_dir_inodes);
+	if (ost->ost_reg_inodes)
+		ocfs2_bitmap_free(&ost->ost_reg_inodes);
+	if (ost->ost_allocated_clusters)
+		ocfs2_bitmap_free(&ost->ost_allocated_clusters);
+	if (ost->ost_duplicate_clusters)
+		ocfs2_bitmap_free(&ost->ost_duplicate_clusters);
+	if (ost->ost_icount_in_inodes)
+		o2fsck_icount_free(ost->ost_icount_in_inodes);
+	if (ost->ost_icount_refs)
+		o2fsck_icount_free(ost->ost_icount_refs);
+}
+
 static errcode_t check_superblock(o2fsck_state *ost)
 {
 	struct ocfs2_dinode *di = ost->ost_fs->fs_super;
@@ -471,6 +489,7 @@ static errcode_t open_and_check(o2fsck_state *ost, char *filename,
 	if (ret) {
 		printf("fsck saw unrecoverable errors in the super block and "
 		       "will not continue.\n");
+		ocfs2_close(ost->ost_fs);
 		goto out;
 	}
 
@@ -517,6 +536,8 @@ static errcode_t maybe_replay_journals(o2fsck_state *ost, char *filename,
 	if (!replayed)
 		goto out;
 
+	fsck_unlock_fs(ost);
+
 	ret = ocfs2_close(ost->ost_fs);
 	if (ret) {
 		com_err(whoami, ret, "while closing \"%s\"", filename);
@@ -524,6 +545,13 @@ static errcode_t maybe_replay_journals(o2fsck_state *ost, char *filename,
 	}
 
 	ret = open_and_check(ost, filename, open_flags, blkno, blksize);
+	if (ret) {
+		com_err(whoami, ret, "while opening and checking \"%s\"",
+				filename);
+		goto out;
+	}
+
+	ret = fsck_lock_fs(ost);
 out:
 	return ret;
 }
@@ -675,6 +703,67 @@ bail:
 	return ret;
 }
 
+static errcode_t fsck_lock_fs(o2fsck_state *ost)
+{
+	errcode_t ret;
+
+	ret = o2cb_init();
+	if (ret) {
+		com_err(whoami, ret, "while initializing the cluster");
+		goto bail;
+	}
+
+	block_signals(SIG_BLOCK);
+	ret = ocfs2_initialize_dlm(ost->ost_fs, whoami);
+	if (ret == O2CB_ET_INVALID_STACK_NAME ||
+			ret == O2CB_ET_INVALID_CLUSTER_NAME ||
+			ret == O2CB_ET_INVALID_HEARTBEAT_MODE) {
+		block_signals(SIG_UNBLOCK);
+		ret = recover_cluster_info(ost);
+		if (ret) {
+			com_err(whoami, ret,
+					"while recovering cluster information");
+			goto bail;
+		}
+		block_signals(SIG_BLOCK);
+		ret = ocfs2_initialize_dlm(ost->ost_fs, whoami);
+	}
+	if (ret) {
+		block_signals(SIG_UNBLOCK);
+		com_err(whoami, ret, "while initializing the DLM");
+		goto bail;
+	}
+
+	ret = ocfs2_lock_down_cluster(ost->ost_fs);
+	if (ret) {
+		if (ost->ost_fs->fs_dlm_ctxt)
+			ocfs2_shutdown_dlm(ost->ost_fs, whoami);
+		block_signals(SIG_UNBLOCK);
+		com_err(whoami, ret, "while locking down the cluster");
+		goto bail;
+	}
+	cluster_locked = 1;
+	block_signals(SIG_UNBLOCK);
+
+bail:
+	return ret;
+}
+
+static void fsck_unlock_fs(o2fsck_state *ost)
+{
+	if (!ost->ost_fs)
+		return;
+
+	block_signals(SIG_BLOCK);
+	if (ost->ost_fs->fs_dlm_ctxt)
+		ocfs2_release_cluster(ost->ost_fs);
+	cluster_locked = 0;
+
+	if (ost->ost_fs->fs_dlm_ctxt)
+		ocfs2_shutdown_dlm(ost->ost_fs, whoami);
+	block_signals(SIG_UNBLOCK);
+}
+
 int main(int argc, char **argv)
 {
 	char *filename;
@@ -845,10 +934,13 @@ int main(int argc, char **argv)
 			fprintf(stdout, "\nWARNING!!! Running fsck.ocfs2 (read-"
 				"only) on a mounted filesystem may detect "
 				"invalid errors.\n\n");
-		else
-			fprintf(stdout, "\nWARNING!!! Running fsck.ocfs2 on a "
+		else {
+			fprintf(stdout, "\nRunning fsck.ocfs2 on a "
 				"mounted filesystem may cause SEVERE "
-				"filesystem damage.\n\n");
+				"filesystem damage, abort.\n\n");
+			fsck_mask |= FSCK_CANCELED;
+			goto out;
+		}
 		proceed = 0;
 	}
 
@@ -898,41 +990,11 @@ int main(int argc, char **argv)
 
 	if (open_flags & OCFS2_FLAG_RW && !ost->ost_skip_o2cb &&
 	    !ocfs2_mount_local(ost->ost_fs)) {
-		ret = o2cb_init();
+		ret = fsck_lock_fs(ost);
 		if (ret) {
-			com_err(whoami, ret, "while initializing the cluster");
+			fsck_mask |= FSCK_ERROR;
 			goto close;
 		}
-
-		block_signals(SIG_BLOCK);
-		ret = ocfs2_initialize_dlm(ost->ost_fs, whoami);
-		if (ret == O2CB_ET_INVALID_STACK_NAME ||
-		    ret == O2CB_ET_INVALID_CLUSTER_NAME ||
-		    ret == O2CB_ET_INVALID_HEARTBEAT_MODE) {
-			block_signals(SIG_UNBLOCK);
-			ret = recover_cluster_info(ost);
-			if (ret) {
-				com_err(whoami, ret,
-					"while recovering cluster information");
-				goto close;
-			}
-			block_signals(SIG_BLOCK);
-			ret = ocfs2_initialize_dlm(ost->ost_fs, whoami);
-		}
-		if (ret) {
-			block_signals(SIG_UNBLOCK);
-			com_err(whoami, ret, "while initializing the DLM");
-			goto close;
-		}
-
-		ret = ocfs2_lock_down_cluster(ost->ost_fs);
-		if (ret) {
-			block_signals(SIG_UNBLOCK);
-			com_err(whoami, ret, "while locking down the cluster");
-			goto close;
-		}
-		cluster_locked = 1;
-		block_signals(SIG_UNBLOCK);
 	}
 
 	printf("Checking OCFS2 filesystem in %s:\n", filename);
@@ -955,6 +1017,7 @@ int main(int argc, char **argv)
 		if (ret) {
 			printf("fsck saw unrecoverable errors in the journal "
 				"files and will not continue.\n");
+			fsck_mask |= FSCK_ERROR;
 			goto unlock;
 		}
 	}
@@ -1082,24 +1145,19 @@ clear_dirty_flag:
 	}
 
 unlock:
-	block_signals(SIG_BLOCK);
-	if (ost->ost_fs->fs_dlm_ctxt)
-		ocfs2_release_cluster(ost->ost_fs);
-	cluster_locked = 0;
-	block_signals(SIG_UNBLOCK);
+	fsck_unlock_fs(ost);
 
 close:
-	block_signals(SIG_BLOCK);
-	if (ost->ost_fs->fs_dlm_ctxt)
-		ocfs2_shutdown_dlm(ost->ost_fs, whoami);
-	block_signals(SIG_UNBLOCK);
+	if (ost->ost_fs) {
+		ret = ocfs2_close(ost->ost_fs);
+		if (ret) {
+			com_err(whoami, ret, "while closing file \"%s\"", filename);
+			/* XXX I wonder about this error.. */
+			fsck_mask |= FSCK_ERROR;
+		}
+	}
 
-	ret = ocfs2_close(ost->ost_fs);
-	if (ret) {
-		com_err(whoami, ret, "while closing file \"%s\"", filename);
-		/* XXX I wonder about this error.. */
-		fsck_mask |= FSCK_ERROR;
-	} 
+	o2fsck_state_release(ost);
 
 out:
 	return fsck_mask;
