@@ -339,6 +339,7 @@ static errcode_t replay_blocks(ocfs2_filesys *fs, struct journal_info *ji,
 			tagp += 16;
 	}
 	
+	*next_block = jwrap(ji->ji_jsb, *next_block);
 out:
 	if (io_buf)
 		ocfs2_free(&io_buf);
@@ -722,7 +723,7 @@ out:
 	if (buf)
 		ocfs2_free(&buf);
 	if (used_blocks)
-		ocfs2_bitmap_free(used_blocks);
+		ocfs2_bitmap_free(&used_blocks);
 
 	return ret;
 }
@@ -925,10 +926,12 @@ static errcode_t check_journal_walk(o2fsck_state *ost,
 		ret = func(ost, ci, jc);
 		if (ret)
 			break;
+
+		ocfs2_free_cached_inode(fs, ci);
+		ci = NULL;
 	}
 
-	if (ci)
-		ocfs2_free_cached_inode(fs, ci);
+	ocfs2_free_cached_inode(fs, ci);
 
 	return ret;
 }
