@@ -1008,7 +1008,8 @@ static errcode_t verify_chain_alloc(o2fsck_state *ost,
 			free += cs.cs_free_bits;
 			total += cs.cs_total_bits;
 			continue;
-		}
+		} else if (allowed && forbidden)
+			continue;
 
 		if (prompt(ost, PY, PR_CHAIN_EMPTY,
 			   "Chain %d in allocator inode %"PRIu64" "
@@ -1293,9 +1294,9 @@ static errcode_t verify_bitmap_descs(o2fsck_state *ost,
 
 out:
 	if (allowed)
-		ocfs2_bitmap_free(allowed);
+		ocfs2_bitmap_free(&allowed);
 	if (forbidden)
-		ocfs2_bitmap_free(forbidden);
+		ocfs2_bitmap_free(&forbidden);
 	return ret;
 }
 
