@@ -299,7 +299,6 @@ static errcode_t dup_insert(struct dup_context *dct, uint32_t cluster,
 	di = dup_inode_lookup(dct, dinode->i_blkno);
 	if (!di) {
 		dup_inode_insert(dct, new_di);
-		di = new_di;
 		new_di = NULL;
 	}
 
@@ -312,7 +311,6 @@ static errcode_t dup_insert(struct dup_context *dct, uint32_t cluster,
 	}
 	if (!dco) {
 		list_add_tail(&new_dco->dco_list, &dc->dc_owners);
-		dco = new_dco;
 		new_dco = NULL;
 	}
 
@@ -969,6 +967,10 @@ static void walk_cwd(struct dir_scan_context *scan)
 			- OCFS2_MAX_FILENAME_LEN;
 
 	ret = ocfs2_malloc(len, &de);
+	if (ret) {
+		pass1c_warn(ret);
+		return;
+	}
 
 	memcpy(de->name, scan->ds_cwd, scan->ds_cwdlen);
 	de->name_len = scan->ds_cwdlen;
