@@ -68,7 +68,8 @@ static ocfs2_fs_options feature_level_defaults[] = {
 	 OCFS2_FEATURE_INCOMPAT_XATTR |
 	 OCFS2_FEATURE_INCOMPAT_REFCOUNT_TREE |
 	 OCFS2_FEATURE_INCOMPAT_INDEXED_DIRS |
-	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG,
+	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG |
+	 OCFS2_FEATURE_INCOMPAT_APPEND_DIO,
 	 OCFS2_FEATURE_RO_COMPAT_UNWRITTEN},  /* OCFS2_FEATURE_LEVEL_DEFAULT */
 
 	{OCFS2_FEATURE_COMPAT_BACKUP_SB | OCFS2_FEATURE_COMPAT_JBD2_SB,
@@ -83,7 +84,8 @@ static ocfs2_fs_options feature_level_defaults[] = {
 	 OCFS2_FEATURE_INCOMPAT_XATTR |
 	 OCFS2_FEATURE_INCOMPAT_REFCOUNT_TREE |
 	 OCFS2_FEATURE_INCOMPAT_INDEXED_DIRS |
-	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG,
+	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG |
+	 OCFS2_FEATURE_INCOMPAT_APPEND_DIO,
 	 OCFS2_FEATURE_RO_COMPAT_UNWRITTEN |
 	 OCFS2_FEATURE_RO_COMPAT_USRQUOTA |
 	 OCFS2_FEATURE_RO_COMPAT_GRPQUOTA }, /* OCFS2_FEATURE_LEVEL_MAX_FEATURES */
@@ -97,7 +99,8 @@ static ocfs2_fs_options mkfstypes_features_defaults[] = {
 	 OCFS2_FEATURE_INCOMPAT_XATTR |
 	 OCFS2_FEATURE_INCOMPAT_REFCOUNT_TREE |
 	 OCFS2_FEATURE_INCOMPAT_INDEXED_DIRS |
-	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG,
+	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG |
+	 OCFS2_FEATURE_INCOMPAT_APPEND_DIO,
 	 OCFS2_FEATURE_RO_COMPAT_UNWRITTEN},  /* OCFS2_MKFSTYPE_DEFAULT */
 
 	{OCFS2_FEATURE_COMPAT_BACKUP_SB | OCFS2_FEATURE_COMPAT_JBD2_SB,
@@ -107,7 +110,8 @@ static ocfs2_fs_options mkfstypes_features_defaults[] = {
 	 OCFS2_FEATURE_INCOMPAT_XATTR |
 	 OCFS2_FEATURE_INCOMPAT_REFCOUNT_TREE |
 	 OCFS2_FEATURE_INCOMPAT_INDEXED_DIRS |
-	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG,
+	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG |
+	 OCFS2_FEATURE_INCOMPAT_APPEND_DIO,
 	 OCFS2_FEATURE_RO_COMPAT_UNWRITTEN},  /* OCFS2_MKFSTYPE_DATAFILES */
 
 	{OCFS2_FEATURE_COMPAT_BACKUP_SB | OCFS2_FEATURE_COMPAT_JBD2_SB,
@@ -117,7 +121,8 @@ static ocfs2_fs_options mkfstypes_features_defaults[] = {
 	 OCFS2_FEATURE_INCOMPAT_XATTR |
 	 OCFS2_FEATURE_INCOMPAT_REFCOUNT_TREE |
 	 OCFS2_FEATURE_INCOMPAT_INDEXED_DIRS |
-	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG,
+	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG |
+	 OCFS2_FEATURE_INCOMPAT_APPEND_DIO,
 	 OCFS2_FEATURE_RO_COMPAT_UNWRITTEN},  /* OCFS2_MKFSTYPE_MAIL */
 
 	{OCFS2_FEATURE_COMPAT_BACKUP_SB | OCFS2_FEATURE_COMPAT_JBD2_SB,
@@ -127,7 +132,8 @@ static ocfs2_fs_options mkfstypes_features_defaults[] = {
 	 OCFS2_FEATURE_INCOMPAT_XATTR |
 	 OCFS2_FEATURE_INCOMPAT_REFCOUNT_TREE |
 	 OCFS2_FEATURE_INCOMPAT_INDEXED_DIRS |
-	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG,
+	 OCFS2_FEATURE_INCOMPAT_DISCONTIG_BG |
+	 OCFS2_FEATURE_INCOMPAT_APPEND_DIO,
 	 OCFS2_FEATURE_RO_COMPAT_UNWRITTEN},  /* OCFS2_MKFSTYPE_VMSTORE */
 };
 
@@ -204,6 +210,11 @@ static struct fs_feature_flags ocfs2_supported_features[] = {
 		{0, OCFS2_FEATURE_INCOMPAT_CLUSTERINFO, 0},
 		{0, OCFS2_FEATURE_INCOMPAT_CLUSTERINFO, 0},
 	},
+	{
+		"append-dio",
+		{0, OCFS2_FEATURE_INCOMPAT_APPEND_DIO, 0},
+		{0, OCFS2_FEATURE_INCOMPAT_APPEND_DIO, 0},
+	},
 	{
 		NULL,
 		{0, 0, 0},
@@ -295,6 +306,10 @@ static struct feature_name ocfs2_feature_names[] = {
 		.fn_name = "clusterinfo",
 		.fn_flag = {0, OCFS2_FEATURE_INCOMPAT_CLUSTERINFO, 0},
 	},
+	{
+		.fn_name = "append-dio",
+		.fn_flag = {0, OCFS2_FEATURE_INCOMPAT_APPEND_DIO, 0},
+	},
 	{
 		.fn_name = NULL,
 	},
