@@ -21,6 +21,7 @@
 #include "mkfs.h"
 
 static State *get_state(int argc, char **argv);
+static void free_state(State *s);
 static int get_number(char *arg, uint64_t *res);
 static void parse_journal_opts(char *progname, const char *opts,
 			       uint64_t *journal_size_in_bytes,
@@ -46,6 +47,7 @@ static int alloc_from_bitmap(State *s, uint64_t num_bits, AllocBitmap *bitmap,
 			     uint64_t *start, uint64_t *num);
 static uint64_t alloc_inode(State *s, uint16_t *suballoc_bit);
 static DirData *alloc_directory(State *s);
+static void free_directory(DirData *dir);
 static void add_entry_to_directory(State *s, DirData *dir, char *name,
 				   uint64_t byte_off, uint8_t type);
 static uint32_t blocks_needed(State *s);
@@ -76,6 +78,7 @@ static AllocGroup * initialize_alloc_group(State *s, const char *name,
 					   uint64_t blkno,
 					   uint16_t chain, uint16_t cpg,
 					   uint16_t bpc);
+static void free_alloc_group(AllocGroup *group);
 static void index_system_dirs(State *s, ocfs2_filesys *fs);
 static void create_lost_found_dir(State *s, ocfs2_filesys *fs);
 static void format_journals(State *s, ocfs2_filesys *fs);
@@ -173,7 +176,6 @@ static int is_cluster_info_valid(State *s, char *stack_name,
 				"the specified cluster attribute");
 			return 0;
 		}
-		return 1;
 	}
 
 	if (!stack_name || !strlen(stack_name)) {
@@ -580,12 +582,14 @@ main(int argc, char **argv)
 	/* bail if volume already mounted on cluster, etc. */
 	switch (ocfs2_check_volume(s)) {
 	case -1:
+		free_state(s);
 		return 1;
 	case 1:
 		if (s->prompt) {
 			fprintf(stdout, "Proceed (y/N): ");
 			if (toupper(getchar()) != 'Y') {
 				printf("Aborting operation.\n");
+				free_state(s);
 				return 1;
 			}
 		}
@@ -607,6 +611,7 @@ main(int argc, char **argv)
 
 	if (s->dry_run) {
 		close_device(s);
+		free_state(s);
 		return 0;
 	}
 
@@ -823,6 +828,16 @@ main(int argc, char **argv)
 	if (!s->quiet)
 		printf("%s successful\n\n", s->progname);
 
+	free_directory(root_dir);
+	free_directory(system_dir);
+	for (i = 0; i < s->initial_slots; i++)
+		free_directory(orphan_dir[i]);
+
+	for (i = 0; i < NUM_SYSTEM_INODES; i++)
+		ocfs2_free(&record[i]);
+
+	free_state(s);
+
 	return 0;
 }
 
@@ -883,7 +898,7 @@ get_state(int argc, char **argv)
 		{ "verbose", 0, 0, 'v' },
 		{ "quiet", 0, 0, 'q' },
 		{ "version", 0, 0, 'V' },
-		{ "journal-options", 0, 0, 'J'},
+		{ "journal-options", 1, 0, 'J'},
 		{ "heartbeat-device", 0, 0, 'H'},
 		{ "force", 0, 0, 'F'},
 		{ "mount", 1, 0, 'M'},
@@ -899,8 +914,6 @@ get_state(int argc, char **argv)
 
 	if (argc && *argv)
 		progname = basename(argv[0]);
-	else
-		progname = strdup("mkfs.ocfs2");
 
 	while (1) {
 		c = getopt_long(argc, argv, "b:C:L:N:J:M:vnqVFHxT:U:",
@@ -1227,6 +1240,28 @@ get_state(int argc, char **argv)
 	return s;
 }
 
+static void
+free_state(State *s)
+{
+	int i;
+
+	ocfs2_free(&s->vol_label);
+	ocfs2_free(&s->device_name);
+	ocfs2_free(&s->cluster_stack);
+	ocfs2_free(&s->cluster_name);
+
+	if (s->global_bm) {
+		for (i = 0; i < s->nr_cluster_groups; i++)
+			free_alloc_group(s->global_bm->groups[i]);
+		ocfs2_free(&s->global_bm->groups);
+		ocfs2_free(&s->global_bm->name);
+		ocfs2_free(&s->global_bm);
+	}
+
+	free_alloc_group(s->system_group);
+	ocfs2_free(&s);
+}
+
 static int
 get_number(char *arg, uint64_t *res)
 {
@@ -1415,10 +1450,30 @@ static unsigned int journal_size_vmstore(State *s)
 	return 32768;
 }
 
+static int journal_size_valid(unsigned int j_blocks, State *s)
+{
+	return (j_blocks * s->initial_slots + 1024) <=
+		s->volume_size_in_blocks;
+}
+
+/* For operations such as mkdir that can require more than a cluster worth
+ * of journal credits, journal size should be greater than cluster size * 8.
+ * The kernel allows the maximum transaction buffer to be 1\4 th of the
+ * journal size and this is further divided by 2 for transaction
+ * reservation support. We calculate minimum journal size here
+ * accordingly and and ceil w.r.t to the cluster size.*/
+static unsigned int journal_min_size(uint32_t cluster_size)
+{
+	return (cluster_size << OCFS2_MIN_CLUSTER_TO_JOURNAL_SIZE_SHIFT)
+		+ cluster_size;
+}
+
 /* stolen from e2fsprogs, modified to fit ocfs2 patterns */
 static uint64_t figure_journal_size(uint64_t size, State *s)
 {
 	unsigned int j_blocks;
+	uint64_t ret;
+	unsigned int min_journal_size;
 
 	if (s->hb_dev)
 		return 0;
@@ -1428,19 +1483,27 @@ static uint64_t figure_journal_size(uint64_t size, State *s)
 		exit(1);
 	}
 
+	min_journal_size = journal_min_size(s->cluster_size);
 	if (size > 0) {
 		j_blocks = size >> s->blocksize_bits;
 		/* mke2fs knows about free blocks at this point, but
 		 * we don't so lets just take a wild guess as to what
 		 * the fs overhead we're looking at will be. */
-		if ((j_blocks * s->initial_slots + 1024) > 
-		    s->volume_size_in_blocks) {
+		if (!journal_size_valid(j_blocks, s)) {
 			fprintf(stderr, 
 				"Journal size too big for filesystem.\n");
 			exit(1);
 		}
 
-		return align_bytes_to_clusters_ceil(s, size);
+		ret = align_bytes_to_clusters_ceil(s, size);
+		/* It is better to fail mkfs than to create a non-functional
+		 * filesystem.*/
+		if (ret < min_journal_size) {
+			fprintf(stderr,
+				"Journal size too small for filesystem.\n");
+			exit(1);
+		}
+		return ret;
 	}
 
 	switch (s->fs_type) {
@@ -1458,7 +1521,22 @@ static uint64_t figure_journal_size(uint64_t size, State *s)
 		break;
 	}
 
-	return align_bytes_to_clusters_ceil(s, j_blocks << s->blocksize_bits);
+	ret = align_bytes_to_clusters_ceil(s, j_blocks << s->blocksize_bits);
+	/* If the default journal size is less than the minimum required
+	 * size, set the default to the minimum size. Then fail if
+	 * the journal size is not valid*/
+	if (ret < min_journal_size) {
+		ret = min_journal_size;
+		j_blocks = ret >> s->blocksize_bits;
+		if (!journal_size_valid(j_blocks, s)) {
+			fprintf(stderr,
+				"Volume size too small for required "
+				"configuration.\nIncrease volume size or "
+				"reduce cluster size\n");
+			exit(1);
+		}
+	}
+	return ret;
 }
 
 static uint32_t cluster_size_default(State *s)
@@ -1841,6 +1919,16 @@ initialize_alloc_group(State *s, const char *name,
 	return group;
 }
 
+static void
+free_alloc_group(AllocGroup *group)
+{
+	if (group) {
+		ocfs2_free(&group->name);
+		ocfs2_free(&group->gd);
+		ocfs2_free(&group);
+	}
+}
+
 static AllocBitmap *
 initialize_bitmap(State *s, uint32_t bits, uint32_t unit_bits,
 		  const char *name, SystemFileDiskRecord *bm_record)
@@ -2128,6 +2216,13 @@ alloc_directory(State *s)
 	return dir;
 }
 
+static void
+free_directory(DirData *dir)
+{
+	ocfs2_free(&dir->buf);
+	ocfs2_free(&dir);
+}
+
 static void
 add_entry_to_directory(State *s, DirData *dir, char *name, uint64_t byte_off,
 		       uint8_t type)
@@ -2797,7 +2892,6 @@ init_record(State *s, SystemFileDiskRecord *rec, int type, int mode)
 {
 	memset(rec, 0, sizeof(SystemFileDiskRecord));
 
-	rec->flags = OCFS2_VALID_FL | OCFS2_SYSTEM_FL;
 	rec->mode = mode;
 
 	rec->links = S_ISDIR(mode) ? 0 : 1;
